import numpy as np

def todo_spectrum(x):
    '''
    Input:
    x (nsamps) - a waveform
    Output:
    spec (nsamps) - magnitude spectrum at nsamps frequencies between 0 and fs, not including fs.
    '''
    spec = np.zeros(len(x),dtype='complex')
    raise NotImplementedError('You need to write this part!')
    return(np.abs(spec))

def todo_findpeak(spec, fs, flo, fhi):
    '''
    Input:
    spec (nsamps) - magnitude spectrum at nsamps frequencies between 0 and fs, not including fs.
    fs (scalar)  - sampling frequency
    flo (scalar) -  low end of the frequency range to search
    fhi (scalar) - high end of the frequency range to search
    Output:
    fpeak (scalar) - frequency of the highest-magnitude spectral sample, in Hertz
    '''
    raise NotImplementedError('You need to write this part!')
    return(fpeak)

def todo_zeros(freqs, fs):
    '''
    Input:
    freqs (nfreqs) - an array of nfreqs Hertz freqeuncies
    fs (scalar) - sampling frequency, in Hertz
    Output:
    z (2,nfreqs) - an array of complex zeros on the unit circle, in complex conjugate pairs
    '''
    nfreqs = len(freqs)
    z = np.zeros((2,nfreqs),dtype='complex')
    raise NotImplementedError('You need to write this part!')
    return(z)

def todo_poles(z,BW,fs):
    '''
    Input: 
    z (2,nfreqs) - an array of complex zeros on the unit circle, in complex conjugate pairs
    BW (scalar) - desired bandwidth, in Hertz
    fs (scalar) - sampling frequency, in Hertz
    Output:
    p (2,nfreqs) - an array of complex poles with bandwidth BW, in complex conjugate pairs
    '''
    nfreqs = z.shape[1]
    p = np.zeros((2,nfreqs),dtype='complex')
    raise NotImplementedError('You need to write this part!')
    return(p)

def todo_coefficients(z):
    '''
    Input: 
    z (2,nfreqs) - an array of complex roots, in complex conjugate pairs
    Output:
    b (3,nfreqs) - an array of second-order polynomial coefficients, one per complex root pair
    '''
    nfreqs = z.shape[1]
    b = np.ones((3,nfreqs))
    raise NotImplementedError('You need to write this part!')
    return(b)

def todo_freqresponse(a, b, nsamps):
    '''
    Input: 
    a (3) - feedback coefficients.  You may assume a[0]=1.
    b (3) - feedforward coefficients.  You may assume b[0]=1.
    nsamps (scalar) - number of samples of the frequency response to compute
    Output: 
    omega (nsamps) - frequencies linearly spaced between 0 and 2pi, not including 2pi.
    H (nsamps) - A(e^{jw})/B(e^{jw}) evaluated at the frequencies in the vector omega.
    '''
    omega = np.zeros(nsamps)
    H = np.zeros(nsamps,dtype='complex')
    raise NotImplementedError('You need to write this part!')
    return(omega, H)
    
def todo_filter(x, a, b):
    '''
    Input: 
    a (3) - feedback coefficients.  You may assume a[0]=1.
    b (3) - feedforward coefficients.  You may assume b[0]=1.
    x (nsamps) - input waveform
    Output: 
    y (nsamps) - output after being filtered using A(z)/B(z)
    '''
    nsamps = len(x)
    y = np.zeros(nsamps)
    raise NotImplementedError('You need to write this part!')
    return(y)

#################################################################
# What to do if somebody calls this from the command line:
if __name__=="__main__":
    import argparse,h5py,wave,struct
    parser = argparse.ArgumentParser(description='''Run MP4 using sample data, and save results.''')
    parser.add_argument('-w','--wav_filename',default='input.wav',
                        help='''wav_filename.  Default: input.wav''')
    parser.add_argument('-f','--fhi',default='150',
                        help='''highest frequency to search for peaks.  Default: 150''')
    parser.add_argument('-b','--bw',default='20',
                        help='''bandwidth of notch filter.  Default: 20Hz''')
    args = parser.parse_args()

    with wave.open(args.wav_filename,'rb') as w:
        fs = w.getframerate()
        nsamps = w.getnframes()
        x = np.frombuffer(w.readframes(nsamps),dtype=np.int16).astype('float32')

    spec=todo_spectrum(x)
    noise1 = todo_findpeak(spec, fs, 0, 100)
    noise2 = todo_findpeak(spec, fs, noise1+1, 100)
    noise3 = todo_findpeak(spec, fs, 100, 110)
    noise4 = todo_findpeak(spec, fs, 100, int(args.fhi))
    freqs = np.array([noise1,noise2,noise3,noise4])    
    z= todo_zeros(freqs, fs)
    p= todo_poles(z,int(args.bw),fs)
    a = todo_coefficients(p)
    b = todo_coefficients(z)
    H = np.ones(nsamps,dtype='complex')
    y = x
    for k in range(len(freqs)):
        omega, Hnew = todo_freqresponse(a[:,k], b[:,k], nsamps)
        H *= Hnew
        y = todo_filter(y, a[:,k], b[:,k])

    with wave.open('results.wav','wb') as w:
        w.setnchannels(1)
        w.setsampwidth(2)
        w.setframerate(fs)
        ymax = np.amax(np.absolute(y))
        for samp in y:
            datum = struct.pack('<h', int(32768*samp/(ymax+1e-6)))
            w.writeframesraw(datum)
        w.close()
        
    with h5py.File('results.hdf5','w') as f:
        f.create_dataset('x',data=x)
        f.create_dataset('fs',data=np.array([fs]))
        f.create_dataset('spec',data=spec)
        f.create_dataset('freqs',data=freqs)
        f.create_dataset('z',data=z)
        f.create_dataset('p',data=p)
        f.create_dataset('a',data=a)
        f.create_dataset('b',data=b)
        f.create_dataset('H',data=H)
        f.create_dataset('y',data=y)
        
    print('mp4 completed successfully; results saved in results.hdf5')
    print('Now try running mp4overview.ipynb, or run_tests.py')

    
